package org.jboss.qa.brms.performance;

import org.jboss.qa.brms.performance.domain.NewCustomerEvent;
import org.jboss.qa.brms.performance.domain.TransactionCreatedEvent;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.rule.SessionEntryPoint;
import org.kie.internal.utils.KieHelper;

import java.util.ArrayList;
import java.util.List;

public class SimpleAfterBeforeOperatorsTest {
    private static int LOOPS = 20;
    private static int NR_OF_EVENTS = 1000;

    private KieBase kieBase;

    @Test
    public void testCreateKBaseInsertFactsAndFireRules() {
        createKBase("fusion-after-before-operators-100.drl");
        for (int i = 0; i < LOOPS; i++) {
           executeOneLoop(i);
        }
    }

    private void createKBase(String drlName) {
        KieHelper kieHelper = new KieHelper();
        kieHelper.addFromClassPath("/" + drlName);
        kieBase = kieHelper.build();
    }

    private void executeOneLoop(int loopNumber) {
        KieSession kieSession = kieBase.newKieSession();
        SessionEntryPoint entryPoint = kieSession.getEntryPoint("Master Stream");
        long start = System.nanoTime();
        for (Object event : getEvents(NR_OF_EVENTS)) {
            entryPoint.insert(event);
        }
        kieSession.fireAllRules();
        long took = System.nanoTime() - start;
        System.out.println("Loop #" + loopNumber + "took " + took / 1000 / 1000 + "ms.");
        kieSession.dispose();
        System.gc();
    }

    private List<Object> getEvents(int nrOfEvents) {
        List<Object> events = new ArrayList<Object>();
        for (int i = 0; i < nrOfEvents / 2; i++) {
            NewCustomerEvent customerEvent = new NewCustomerEvent(i);
            customerEvent.setDescription("AfterBefore_TransactionCreatedAfterNewCustomerCreated");
            events.add(customerEvent);
            TransactionCreatedEvent transactionCreatedEvent = new TransactionCreatedEvent(i);
            transactionCreatedEvent.setDescription("AfterBefore_TransactionCreatedBeforeNewCustomerCreated");
            events.add(transactionCreatedEvent);
        }
        return events;
    }


}
